from jsonschema import validators, ValidationError
from jsonschema import validate as jsonschema_validate

def extend_with_default(validator_class):
    validate_properties = validator_class.VALIDATORS["properties"]

    def set_defaults(validator, properties, instance, schema):
        if isinstance(instance, bool):
            return
        for property, subschema in properties.items():
            if isinstance(subschema, bool):
                continue
            if "default" in subschema:
                instance.setdefault(property, subschema["default"])

    return validators.extend(
        validator_class, {"properties" : set_defaults},
    )


def validate(instance, schema, cls=None, *args, **kwargs):
    validator_class = validators.validator_for(schema)
    validator_class_with_defaults = extend_with_default(validator_class)
    return jsonschema_validate(instance, schema, validator_class_with_defaults,
                               *args, **kwargs)
